# Unit tests to check your solution
import unittest
from standard_distributions.binomial import Binomial


class TestBinomialClass(unittest.TestCase):
    def setUp(self):
        self.binomial = Binomial(0.4, 20)
        self.file_path = "../../resources/numbers_binomial.txt"
        self.binomial.read_data_file(self.file_path)

    def test_initialization(self):
        self.assertEqual(self.binomial.probability, 0.4, "p value incorrect")
        self.assertEqual(self.binomial.size, 20, "n value incorrect")

    def test_readdata(self):
        self.assertEqual(
            self.binomial.data,
            [0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0],
            "data not read in correctly",
        )

    def test_calculate_mean(self):
        mean = self.binomial.calculate_mean()
        self.assertEqual(mean, 8)

    def test_calculate_stdev(self):
        stdev = self.binomial.calculate_stdev()
        self.assertEqual(round(stdev, 2), 2.19)

    def test_replace_stats_with_data(self):
        p, n = self.binomial.replace_stats_with_data()
        self.assertEqual(round(p, 3), 0.615)
        self.assertEqual(n, 13)

    def test_pdf(self):
        self.assertEqual(round(self.binomial.pdf(5), 5), 0.07465)
        self.assertEqual(round(self.binomial.pdf(3), 5), 0.01235)

        self.binomial.replace_stats_with_data()
        self.assertEqual(round(self.binomial.pdf(5), 5), 0.05439)
        self.assertEqual(round(self.binomial.pdf(3), 5), 0.00472)

    def test_add(self):
        binomial_one = Binomial(0.4, 20)
        binomial_two = Binomial(0.4, 60)
        binomial_sum = binomial_one + binomial_two

        self.assertEqual(binomial_sum.probability, 0.4)
        self.assertEqual(binomial_sum.size, 80)


tests = TestBinomialClass()

tests_loaded = unittest.TestLoader().loadTestsFromModule(tests)

unittest.TextTestRunner().run(tests_loaded)



import math
import matplotlib.pyplot as plt
from standard_distributions import Distribution


class Binomial(Distribution):
    """ Binomial distribution class for calculating and visualizing a Binomial distribution.

    Attributes:
        mean (float) representing the mean value of the distribution
        stdev (float) representing the standard deviation of the distribution
        data_list (list of floats) a list of floats to be extracted from the data file
        probability (float) representing the probability of an event occurring
        size (int) number of trials

    """

    def __init__(self, probability=0.5, size=20):
        self.size = size
        self.probability = probability

        super(Binomial, self).__init__(self.calculate_mean(), self.calculate_stdev())

    def calculate_mean(self):
        """Function to calculate the mean from p and n

            Args:
                None

            Returns:
                float: mean of the data set

        """

        self.mean = self.probability * self.size

        return self.mean

    def calculate_stdev(self):
        """Function to calculate the standard deviation from p and n.

            Args:
                None

            Returns:
                float: standard deviation of the data set

        """

        self.stdev = math.sqrt(self.size * self.probability * (1 - self.probability))

        return self.stdev

    def replace_stats_with_data(self):
        """Function to calculate p and n from the data set

        Args:
            None

        Returns:
            float: the p value
            float: the n value

        """

        self.size = len(self.data)
        self.probability = 1.0 * sum(self.data) / len(self.data)
        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev()

        return self.probability, self.size

    def plot_bar(self):
        """Function to output a histogram of the instance variable data using
        matplotlib pyplot library.

        Args:
            None

        Returns:
            None
        """

        plt.bar(
            x=["0", "1"],
            height=[(1 - self.probability) * self.size, self.probability * self.size],
        )
        plt.title("Bar Chart of Data")
        plt.xlabel("outcome")
        plt.ylabel("count")

    def pdf(self, k):
        """Probability density function calculator for the binomial distribution.

        Args:
            k (float): point for calculating the probability density function


        Returns:
            float: probability density function output
        """

        a = math.factorial(self.size) / (
            math.factorial(k) * (math.factorial(self.size - k))
        )
        b = (self.probability ** k) * (1 - self.probability) ** (self.size - k)

        return a * b

    def plot_bar_pdf(self):

        """Function to plot the pdf of the binomial distribution

        Args:
            None

        Returns:
            list: x values for the pdf plot
            list: y values for the pdf plot

        """

        x = []
        y = []

        # calculate the x values to visualize
        for i in range(self.size + 1):
            x.append(i)
            y.append(self.pdf(i))

        # make the plots
        plt.bar(x, y)
        plt.title("Distribution of Outcomes")
        plt.ylabel("Probability")
        plt.xlabel("Outcome")

        plt.show()

        return x, y

    def __add__(self, other):

        """Function to add together two Binomial distributions with equal p

        Args:
            other (Binomial): Binomial instance

        Returns:
            Binomial: Binomial distribution

        """

        try:
            assert (
                self.probability == other.probability
            ), "probability values are not equal"
        except AssertionError as error:
            raise

        result = Binomial()
        result.data = self.data + other.data
        result.size = self.size + other.size
        result.probability = self.probability
        result.calculate_mean()
        result.calculate_stdev()

        return result

    def __repr__(self):

        """Function to output the characteristics of the Binomial instance

        Args:
            None

        Returns:
            string: characteristics of the Gaussian

        """

        return "mean {}, standard deviation {}, probability {}, size {}".format(
            self.mean, self.stdev, self.probability, self.size
        )



import math
import matplotlib.pyplot as plt
from standard_distributions import Distribution


class Gaussian(Distribution):
    """ Gaussian distribution class for calculating and visualizing a Gaussian distribution.
    
    Attributes:
        mean (float) representing the mean value of the distribution
        stdev (float) representing the standard deviation of the distribution
        data (list of floats) a list of floats extracted from the data file
            
    """

    def __init__(self, mu=0, sigma=1):
        super(Gaussian, self).__init__(mu, sigma)

    def calculate_mean(self):

        """Method to calculate the mean of the data set.
        
        Args: 
            None
        
        Returns: 
            float: mean of the data set
    
        """
        # to avoid ZeroDivisionError exceptions,
        # we always make sure the divisor is greater than 0
        self.mean = 1.0 * sum(self.data) / max(len(self.data), 1)
        return self.mean

    def calculate_stdev(self, sample=True):

        """Method to calculate the standard deviation of the data set.
        
        Args: 
            sample (bool): whether the data represents a sample or population
        
        Returns: 
            float: standard deviation of the data set
    
        """
        count = len(self.data)
        if sample:
            count -= 1

        sigma = sum([(d - self.mean) ** 2 for d in self.data])

        self.stdev = math.sqrt(sigma / count)

        return self.stdev

    def read_data_file(self, file_name, sample=True):

        """Method to read in data from a txt file. The txt file should have
        one number (float) per line. The numbers are stored in the data attribute. 
        After reading in the file, the mean and standard deviation are calculated
                
        Args:
            file_name (string): name of a file to read from
            sample (bool): whether the data represents a sample or population
        
        Returns:
            None
        
        """
        super(Gaussian, self).read_data_file(file_name=file_name)

        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev(sample)

    def plot_histogram(self):
        """Method to output a histogram of the instance variable data using
        matplotlib pyplot library.
        
        Args:
            None
            
        Returns:
            None
        """

        plt.hist(self.data)
        plt.title("Histogram of Data")
        plt.xlabel("data")
        plt.ylabel("count")

    def pdf(self, x):
        """Probability density function calculator for the gaussian distribution.
        
        Args:
            x (float): point for calculating the probability density function
            
        
        Returns:
            float: probability density function output
        """

        return (1.0 / (self.stdev * math.sqrt(2 * math.pi))) * math.exp(
            -0.5 * ((x - self.mean) / self.stdev) ** 2
        )

    def plot_histogram_pdf(self, n_spaces=50):

        """Method to plot the normalized histogram of the data and a plot of the 
        probability density function along the same range
        
        Args:
            n_spaces (int): number of data points 
        
        Returns:
            list: x values for the pdf plot
            list: y values for the pdf plot
            
        """

        mu = self.mean
        sigma = self.stdev

        min_range = min(self.data)
        max_range = max(self.data)

        # calculates the interval between x values
        interval = 1.0 * (max_range - min_range) / n_spaces

        x = []
        y = []

        # calculate the x values to visualize
        for i in range(n_spaces):
            tmp = min_range + interval * i
            x.append(tmp)
            y.append(self.pdf(tmp))

        # make the plots
        fig, axes = plt.subplots(2, sharex=True)
        fig.subplots_adjust(hspace=0.5)
        axes[0].hist(self.data, density=True)
        axes[0].set_title("Normed Histogram of Data")
        axes[0].set_ylabel("Density")

        axes[1].plot(x, y)
        axes[1].set_title(
            "Normal Distribution for \n Sample Mean and Sample Standard Deviation"
        )
        axes[0].set_ylabel("Density")
        plt.show()

        return x, y

    def __add__(self, other):

        """Magic method to add together two Gaussian distributions
        
        Args:
            other (Gaussian): Gaussian instance
            
        Returns:
            Gaussian: Gaussian distribution
            
        """

        result = Gaussian()

        result.data = self.data + other.data
        result.mean = self.mean + other.mean
        result.stdev = math.sqrt((self.stdev ** 2) + (other.stdev ** 2))

        return result

    def __repr__(self):

        """Magic method to output the characteristics of the Gaussian instance
        
        Args:
            None
        
        Returns:
            string: characteristics of the Gaussian
        
        """

        return "mean {0}, standard deviation {1}".format(self.mean, self.stdev)



# Unit tests to check your solution
import unittest
from standard_distributions.gaussian import Gaussian


class TestGaussianClass(unittest.TestCase):
    def setUp(self):
        self.gaussian = Gaussian(25, 2)
        self.file_path = "../../resources/numbers.txt"

    def test_initialization(self):
        self.assertEqual(self.gaussian.mean, 25, "incorrect mean")
        self.assertEqual(self.gaussian.stdev, 2, "incorrect standard deviation")

    def test_pdf(self):
        self.assertEqual(
            round(self.gaussian.pdf(25), 5),
            0.19947,
            "pdf function does not give expected result",
        )

    def test_mean_calculation(self):
        self.gaussian.read_data_file(self.file_path, True)
        self.gaussian.plot_histogram()
        self.assertEqual(
            self.gaussian.calculate_mean(),
            sum(self.gaussian.data) / float(max(1, len(self.gaussian.data))),
            "calculated mean not as expected",
        )

    def test_stdev_calculation(self):
        self.gaussian.read_data_file(self.file_path, True)
        self.assertEqual(
            round(self.gaussian.stdev, 2), 92.87, "sample standard deviation incorrect"
        )
        self.gaussian.read_data_file(self.file_path, False)
        self.assertEqual(
            round(self.gaussian.stdev, 2),
            88.55,
            "population standard deviation incorrect",
        )

    def test_add(self):
        gaussian_one = Gaussian(25, 3)
        gaussian_two = Gaussian(30, 4)
        gaussian_sum = gaussian_one + gaussian_two

        self.assertEqual(gaussian_sum.mean, 55)
        self.assertEqual(gaussian_sum.stdev, 5)

    def test_repr(self):
        gaussian_one = Gaussian(25, 3)

        self.assertEqual(str(gaussian_one), "mean 25, standard deviation 3")


tests = TestGaussianClass()

tests_loaded = unittest.TestLoader().loadTestsFromModule(tests)

unittest.TextTestRunner().run(tests_loaded)



import setuptools

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name="standard_distributions",
    version="0.0.1",
    description="Probability distributions package",
    url="",
    author="Kelvin Wahome",
    author_email="kevowahome@gmail.com",
    license="MIT",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    zip_safe=False,
)
